//
//  EAGLView.m
//  3dMaze
//
//  Created by Ay on 09年2月2日.
//  Copyright NA 2009. All rights reserved.
//



#import <QuartzCore/QuartzCore.h>
#import <OpenGLES/EAGLDrawable.h>

#import "EAGLView.h"



#define USE_DEPTH_BUFFER 1

// A class extension to declare private methods
@interface EAGLView ()

@property (nonatomic, retain) EAGLContext *context;
@property (nonatomic, assign) NSTimer *animationTimer;

- (BOOL) createFramebuffer;
- (void) destroyFramebuffer;

@end


@implementation EAGLView

@synthesize context;
@synthesize animationTimer;
@synthesize animationInterval;


// You must implement this
+ (Class)layerClass {
	return [CAEAGLLayer class];
}


//The GL view is stored in the nib file. When it's unarchived it's sent -initWithCoder:
- (id)initWithCoder:(NSCoder*)coder {

	if ((self = [super initWithCoder:coder])) {
		// Get the layer
		CAEAGLLayer *eaglLayer = (CAEAGLLayer *)self.layer;
		
		eaglLayer.opaque = YES;
		eaglLayer.drawableProperties = [NSDictionary dictionaryWithObjectsAndKeys:
		   [NSNumber numberWithBool:NO], kEAGLDrawablePropertyRetainedBacking, kEAGLColorFormatRGBA8, kEAGLDrawablePropertyColorFormat, nil];
		
		context = [[EAGLContext alloc] initWithAPI:kEAGLRenderingAPIOpenGLES1];
		
		if (!context || ![EAGLContext setCurrentContext:context]) {
			[self release];
			return nil;
		}
		
		animationInterval = 1.0 / 60.0;
	}
	return self;
}


- (void)drawView {
	
	// Replace the implementation of this method to do your own custom drawing
	[EAGLContext setCurrentContext:context];
	
	glBindFramebufferOES(GL_FRAMEBUFFER_OES, viewFramebuffer);
	
	glClear(GL_COLOR_BUFFER_BIT| GL_DEPTH_BUFFER_BIT);
	
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	
	glEnable(GL_TEXTURE_2D);
	
	
	[model render];
	[img render:130 y:100 angle:-45 xScale:1 yScale:1 r:1 g:1 b:1 a:0.5];
	[img render:130 y:400 angle:-135 xScale:-1 yScale:1 r:1 g:1 b:1 a:0.5];
	[img2 render:20 y:240 angle:0 xScale:0.5 yScale:1 r:1 g:1 b:1 a:0.5];
	[img2 render:300 y:240 angle:0 xScale:-0.5 yScale:1 r:1 g:1 b:1 a:0.5];

	
	glBindRenderbufferOES(GL_RENDERBUFFER_OES, viewRenderbuffer);
	[context presentRenderbuffer:GL_RENDERBUFFER_OES];
}

-  (void) viewOperation:(NSSet *)touches{
	startLocation  = [[touches anyObject] locationInView:self];
	if(startLocation.y < 100){
		[self playWalk:[model viewMode:LEFT]];
	}else if(startLocation.y > 300){
		[self playWalk:[model viewMode:RIGHT]];
	}else if(startLocation.x <100){
		[self playWalk:[model viewMode:BACK]];
	}else if(startLocation.x >200){
		[self playWalk:[model viewMode:FRONT]];
	}
	
}

- (void) playWalk:(bool)condi{
	if(condi){
		//[bgSound play];
		[sound play];
		
	}
}
- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event {	
	[self viewOperation:(NSSet *)touches];
}
- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event {
	[self viewOperation:(NSSet *)touches];
	//NSLog(@"X : %f",startLocation.x);
	//NSLog(@"Y : %f",startLocation.y);
}
- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event {
	//[model viewMode:STOP];
}


- (void)layoutSubviews {
	[EAGLContext setCurrentContext:context];
	[self destroyFramebuffer];
	[self createFramebuffer];
	glViewport(0, 0, backingWidth, backingHeight);
	
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrthof(0, (float)backingWidth, 0, (float)backingHeight, 0, 100);
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	
	glEnableClientState (GL_VERTEX_ARRAY);
    glEnableClientState (GL_TEXTURE_COORD_ARRAY);
	
	glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
	
	//glEnable(GL_CULL_FACE);
	model = [[ModelControler alloc] initWithMD2PathAndTexture:@"maze01.md2" texture:@"texture1.png"];
	img = [[imgControler alloc] initWithPath:@"r1.png" width:107 heigh:82];
	img2 = [[imgControler alloc] initWithPath:@"z1.png" width:107 heigh:82];
	
	NSBundle *mainbundle = [NSBundle mainBundle];
	sound = [[SoundEffect alloc] initWithContentsOfFile:[mainbundle pathForResource:@"rw223_150" ofType:@"mp3"]];
	
	//bg music
	bgSound = [[GBMusicTrack alloc] initWithPath:[mainbundle pathForResource:@"A00035" ofType:@"mp3"]];
	[bgSound setRepeat:TRUE];
	[bgSound play];
	
	[self drawView];
}


- (BOOL)createFramebuffer {
	
	glGenFramebuffersOES(1, &viewFramebuffer);
	glGenRenderbuffersOES(1, &viewRenderbuffer);
	
	glBindFramebufferOES(GL_FRAMEBUFFER_OES, viewFramebuffer);
	glBindRenderbufferOES(GL_RENDERBUFFER_OES, viewRenderbuffer);
	[context renderbufferStorage:GL_RENDERBUFFER_OES fromDrawable:(CAEAGLLayer*)self.layer];
	glFramebufferRenderbufferOES(GL_FRAMEBUFFER_OES, GL_COLOR_ATTACHMENT0_OES, GL_RENDERBUFFER_OES, viewRenderbuffer);
	
	glGetRenderbufferParameterivOES(GL_RENDERBUFFER_OES, GL_RENDERBUFFER_WIDTH_OES, &backingWidth);
	glGetRenderbufferParameterivOES(GL_RENDERBUFFER_OES, GL_RENDERBUFFER_HEIGHT_OES, &backingHeight);
	
	if (USE_DEPTH_BUFFER) {
		glGenRenderbuffersOES(1, &depthRenderbuffer);
		glBindRenderbufferOES(GL_RENDERBUFFER_OES, depthRenderbuffer);
		glRenderbufferStorageOES(GL_RENDERBUFFER_OES, GL_DEPTH_COMPONENT16_OES, backingWidth, backingHeight);
		glFramebufferRenderbufferOES(GL_FRAMEBUFFER_OES, GL_DEPTH_ATTACHMENT_OES, GL_RENDERBUFFER_OES, depthRenderbuffer);
	}

	if(glCheckFramebufferStatusOES(GL_FRAMEBUFFER_OES) != GL_FRAMEBUFFER_COMPLETE_OES) {
		NSLog(@"failed to make complete framebuffer object %x", glCheckFramebufferStatusOES(GL_FRAMEBUFFER_OES));
		return NO;
	}
	
	return YES;
}


- (void)destroyFramebuffer {
	
	glDeleteFramebuffersOES(1, &viewFramebuffer);
	viewFramebuffer = 0;
	glDeleteRenderbuffersOES(1, &viewRenderbuffer);
	viewRenderbuffer = 0;
	
	if(depthRenderbuffer) {
		glDeleteRenderbuffersOES(1, &depthRenderbuffer);
		depthRenderbuffer = 0;
	}
}


- (void)startAnimation {
	self.animationTimer = [NSTimer scheduledTimerWithTimeInterval:animationInterval target:self selector:@selector(drawView) userInfo:nil repeats:YES];
}


- (void)stopAnimation {
	self.animationTimer = nil;
}


- (void)setAnimationTimer:(NSTimer *)newTimer {
	[animationTimer invalidate];
	animationTimer = newTimer;
}


- (void)setAnimationInterval:(NSTimeInterval)interval {
	
	animationInterval = interval;
	if (animationTimer) {
		[self stopAnimation];
		[self startAnimation];
	}
}


- (void)dealloc {
	
	[self stopAnimation];
	
	if ([EAGLContext currentContext] == context) {
		[EAGLContext setCurrentContext:nil];
	}
	
	[model dealloc];
	[context release];	
	[super dealloc];
}

@end
